@@ -1,8 +1,8 @@
 #![allow(clippy::exhaustive_structs)]
 
 use ruma_api::{
-    ruma_api, IncomingRequest as _, OutgoingRequest as _, OutgoingRequestAppserviceExt,
-    SendAccessToken,
+    ruma_api, IncomingRequest as _, MatrixVersion, OutgoingRequest as _,
+    OutgoingRequestAppserviceExt, SendAccessToken,
 };
 use ruma_identifiers::{user_id, UserId};
 
@@ -11,7 +11,7 @@ ruma_api! {
         description: "Does something.",
         method: POST,
         name: "my_endpoint",
-        path: "/_matrix/foo/:bar/:user",
+        unstable_path: "/_matrix/foo/:bar/:user",
         rate_limited: false,
         authentication: None,
     }
@@ -27,7 +27,7 @@ ruma_api! {
         #[ruma_api(path)]
         pub bar: String,
         #[ruma_api(path)]
-        pub user: UserId,
+        pub user: Box<UserId>,
     }
 
     response: {
@@ -47,14 +47,18 @@ fn request_serde() {
         q1: "query_param_special_chars %/&@!".to_owned(),
         q2: 55,
         bar: "barVal".to_owned(),
-        user: user_id!("@bazme:ruma.io"),
+        user: user_id!("@bazme:ruma.io").to_owned(),
     };
 
     let http_req = req
         .clone()
-        .try_into_http_request::<Vec<u8>>("https://homeserver.tld", SendAccessToken::None)
+        .try_into_http_request::<Vec<u8>>(
+            "https://homeserver.tld",
+            SendAccessToken::None,
+            &[MatrixVersion::V1_1],
+        )
         .unwrap();
-    let req2 = Request::try_from_http_request(http_req).unwrap();
+    let req2 = Request::try_from_http_request(http_req, &["barVal", "@bazme:ruma.io"]).unwrap();
 
     assert_eq!(req.hello, req2.hello);
     assert_eq!(req.world, req2.world);
@@ -72,10 +76,14 @@ fn invalid_uri_should_not_panic() {
         q1: "query_param_special_chars %/&@!".to_owned(),
         q2: 55,
         bar: "barVal".to_owned(),
-        user: user_id!("@bazme:ruma.io"),
+        user: user_id!("@bazme:ruma.io").to_owned(),
     };
 
-    let result = req.try_into_http_request::<Vec<u8>>("invalid uri", SendAccessToken::None);
+    let result = req.try_into_http_request::<Vec<u8>>(
+        "invalid uri",
+        SendAccessToken::None,
+        &[MatrixVersion::V1_1],
+    );
     assert!(result.is_err());
 }
 
@@ -87,7 +95,7 @@ fn request_with_user_id_serde() {
         q1: "query_param_special_chars %/&@!".to_owned(),
         q2: 55,
         bar: "barVal".to_owned(),
-        user: user_id!("@bazme:ruma.io"),
+        user: user_id!("@bazme:ruma.io").to_owned(),
     };
 
     let user_id = user_id!("@_virtual_:ruma.io");
@@ -96,6 +104,7 @@ fn request_with_user_id_serde() {
             "https://homeserver.tld",
             SendAccessToken::None,
             user_id,
+            &[MatrixVersion::V1_1],
         )
         .unwrap();
 
@@ -108,6 +117,8 @@ fn request_with_user_id_serde() {
 }
 
 mod without_query {
+    use ruma_api::MatrixVersion;
+
     use super::{ruma_api, user_id, OutgoingRequestAppserviceExt, SendAccessToken, UserId};
 
     ruma_api! {
@@ -115,7 +126,7 @@ mod without_query {
             description: "Does something without query.",
             method: POST,
             name: "my_endpoint",
-            path: "/_matrix/foo/:bar/:user",
+            unstable_path: "/_matrix/foo/:bar/:user",
             rate_limited: false,
             authentication: None,
         }
@@ -127,7 +138,7 @@ mod without_query {
             #[ruma_api(path)]
             pub bar: String,
             #[ruma_api(path)]
-            pub user: UserId,
+            pub user: Box<UserId>,
         }
 
         response: {
@@ -145,7 +156,7 @@ mod without_query {
             hello: "hi".to_owned(),
             world: "test".to_owned(),
             bar: "barVal".to_owned(),
-            user: user_id!("@bazme:ruma.io"),
+            user: user_id!("@bazme:ruma.io").to_owned(),
         };
 
         let user_id = user_id!("@_virtual_:ruma.io");
@@ -154,6 +165,7 @@ mod without_query {
                 "https://homeserver.tld",
                 SendAccessToken::None,
                 user_id,
+                &[MatrixVersion::V1_1],
             )
             .unwrap();
 
