@@ -8,4 +8,6 @@ fn ui() {
     t.pass("tests/ui/05-request-only.rs");
     t.pass("tests/ui/06-response-only.rs");
     t.compile_fail("tests/ui/07-error-type-attribute.rs");
+    t.compile_fail("tests/ui/08-deprecated-without-added.rs");
+    t.compile_fail("tests/ui/09-removed-without-deprecated.rs");
 }
