@@ -13,7 +13,7 @@ mod newtype_body {
             description: "Does something.",
             method: POST,
             name: "my_endpoint",
-            path: "/_matrix/foo/:bar/",
+            unstable_path: "/_matrix/foo/:bar/",
             rate_limited: false,
             authentication: None,
         }
@@ -26,7 +26,7 @@ mod newtype_body {
             pub bar: String,
 
             #[ruma_api(query)]
-            pub baz: UserId,
+            pub baz: Box<UserId>,
 
             #[ruma_api(header = CONTENT_TYPE)]
             pub world: String,
@@ -51,7 +51,7 @@ mod raw_body {
             description: "Does something.",
             method: POST,
             name: "my_endpoint",
-            path: "/_matrix/foo/:bar/",
+            unstable_path: "/_matrix/foo/:bar/",
             rate_limited: false,
             authentication: None,
         }
@@ -64,7 +64,7 @@ mod raw_body {
             pub bar: String,
 
             #[ruma_api(query)]
-            pub baz: UserId,
+            pub baz: Box<UserId>,
 
             #[ruma_api(header = CONTENT_TYPE)]
             pub world: String,
@@ -92,7 +92,7 @@ mod plain {
             description: "Does something.",
             method: POST,
             name: "my_endpoint",
-            path: "/_matrix/foo/:bar/",
+            unstable_path: "/_matrix/foo/:bar/",
             rate_limited: false,
             authentication: None,
         }
@@ -100,10 +100,11 @@ mod plain {
         request: {
             pub q2: Foo,
 
+            #[ruma_api(path)]
             pub bar: String,
 
             #[ruma_api(query)]
-            pub baz: UserId,
+            pub baz: Box<UserId>,
 
             #[ruma_api(header = CONTENT_TYPE)]
             pub world: String,
